@@ -1,6 +1,8 @@
 import json
 import sys
 from typing import List, Any, Mapping
+from unittest import mock
+
 import pytest
 
 import sentry_sdk
@@ -268,6 +270,61 @@ def test_metrics_experimental_before_send(sentry_init, capture_envelopes):
     assert before_metric_called
 
 
+def test_transport_format(sentry_init, capture_envelopes):
+    sentry_init(server_name="test-server", release="1.0.0")
+
+    envelopes = capture_envelopes()
+
+    sentry_sdk.metrics.count("test.counter", 1)
+
+    sentry_sdk.get_client().flush()
+
+    assert len(envelopes) == 1
+    assert len(envelopes[0].items) == 1
+    item = envelopes[0].items[0]
+
+    assert item.type == "trace_metric"
+    assert item.headers == {
+        "type": "trace_metric",
+        "item_count": 1,
+        "content_type": "application/vnd.sentry.items.trace-metric+json",
+    }
+    assert item.payload.json == {
+        "items": [
+            {
+                "name": "test.counter",
+                "type": "counter",
+                "value": 1,
+                "timestamp": mock.ANY,
+                "trace_id": mock.ANY,
+                "span_id": mock.ANY,
+                "attributes": {
+                    "sentry.environment": {
+                        "type": "string",
+                        "value": "production",
+                    },
+                    "sentry.release": {
+                        "type": "string",
+                        "value": "1.0.0",
+                    },
+                    "sentry.sdk.name": {
+                        "type": "string",
+                        "value": mock.ANY,
+                    },
+                    "sentry.sdk.version": {
+                        "type": "string",
+                        "value": VERSION,
+                    },
+                    "server.address": {
+                        "type": "string",
+                        "value": "test-server",
+                    },
+                },
+            }
+        ]
+    }
+
+
 def test_batcher_drops_metrics(sentry_init, monkeypatch):
     sentry_init()
     client = sentry_sdk.get_client()
@@ -290,3 +347,85 @@ def test_batcher_drops_metrics(sentry_init, monkeypatch):
     assert len(lost_event_calls) == 5
     for lost_event_call in lost_event_calls:
         assert lost_event_call == ("queue_overflow", "trace_metric", 1)
+
+
+def test_metric_gets_attributes_from_scopes(sentry_init, capture_envelopes):
+    sentry_init()
+
+    envelopes = capture_envelopes()
+
+    global_scope = sentry_sdk.get_global_scope()
+    global_scope.set_attribute("global.attribute", "value")
+
+    with sentry_sdk.new_scope() as scope:
+        scope.set_attribute("current.attribute", "value")
+        sentry_sdk.metrics.count("test", 1)
+
+    sentry_sdk.metrics.count("test", 1)
+
+    get_client().flush()
+
+    metrics = envelopes_to_metrics(envelopes)
+    (metric1, metric2) = metrics
+
+    assert metric1["attributes"]["global.attribute"] == "value"
+    assert metric1["attributes"]["current.attribute"] == "value"
+
+    assert metric2["attributes"]["global.attribute"] == "value"
+    assert "current.attribute" not in metric2["attributes"]
+
+
+def test_metric_attributes_override_scope_attributes(sentry_init, capture_envelopes):
+    sentry_init()
+
+    envelopes = capture_envelopes()
+
+    with sentry_sdk.new_scope() as scope:
+        scope.set_attribute("durable.attribute", "value1")
+        scope.set_attribute("temp.attribute", "value1")
+        sentry_sdk.metrics.count("test", 1, attributes={"temp.attribute": "value2"})
+
+    get_client().flush()
+
+    metrics = envelopes_to_metrics(envelopes)
+    (metric,) = metrics
+
+    assert metric["attributes"]["durable.attribute"] == "value1"
+    assert metric["attributes"]["temp.attribute"] == "value2"
+
+
+def test_attributes_preserialized_in_before_send(sentry_init, capture_envelopes):
+    """We don't surface references to objects in attributes."""
+
+    def before_send_metric(metric, _):
+        assert isinstance(metric["attributes"]["instance"], str)
+        assert isinstance(metric["attributes"]["dictionary"], str)
+
+        return metric
+
+    sentry_init(before_send_metric=before_send_metric)
+
+    envelopes = capture_envelopes()
+
+    class Cat:
+        pass
+
+    instance = Cat()
+    dictionary = {"color": "tortoiseshell"}
+
+    sentry_sdk.metrics.count(
+        "test.counter",
+        1,
+        attributes={
+            "instance": instance,
+            "dictionary": dictionary,
+        },
+    )
+
+    get_client().flush()
+
+    metrics = envelopes_to_metrics(envelopes)
+    (metric,) = metrics
+
+    assert isinstance(metric["attributes"]["instance"], str)
+    assert isinstance(metric["attributes"]["dictionary"], str)
